package Package1;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.TreeMap;

enum BritishNoteValue {

    SEMIBREVE, MINIM, CROTCHET, QUAVER, SEMIQUAVER, DEMISEMIQUAVER, HEMIDEMISEMIQUAVER, SEMIHEMIDEMISEMIQUAVER, DEMISEMIHEMIDEMISEMIQUAVER
}

enum AmericanNoteValue {

    WHOLE, HALF, QUARTER, EIGHTH, SIXTEENTH, THIRTYSECOND, SIXTYFOURTH, HUNDREDTWENTYEIGHTH, TWOHUNDREDFIFTYSIXTH
}

enum PitchEnum {

    A, B, C, D, E, F, G, UNDEFINED
}

enum AccidentalEnum {

    SHARP, FLAT, NATURAL, DOUBLESHARP, DOUBLEFLAT, UNDEFINED
}

// ------------------------------------------------------------------------
class NoteRest {

    protected int numerator;
    protected int denominator;

    public NoteRest() {
        numerator = 1;
        denominator = 1;
    }

    public NoteRest(int n, int expd) {
        numerator = Math.abs(n);
        expd = Math.max(0, expd);
        expd = Math.min(expd, 8);
        denominator = (int) (Math.pow(2, expd)); // pow returns a double
    }

    public NoteRest(int expd) {
        numerator = 1;
        expd = Math.max(0, expd);
        expd = Math.min(expd, 8);
        denominator = (int) (Math.pow(2, expd)); // pow returns a double
    }

    public NoteRest(BritishNoteValue value) {
        numerator = 1;
        denominator = (int) (Math.pow(2, value.ordinal()));
    }

    public NoteRest(AmericanNoteValue value) {
        numerator = 1;
        denominator = (int) (Math.pow(2, value.ordinal()));
    }

    @Override
    public String toString() {
        return ("- [" + numerator + "/" + denominator + "]");
    }

    public void halveValue() {
        denominator *= 2;
    }

    public void doubleValue() {
        if (denominator > 1)
            denominator /= 2;
    }

    public int getNumerator() {
        return numerator;
    }

    public int getDenominator() {
        return denominator;
    }

    public double getNumericDuration() {
        return (double) numerator / denominator;
    }
}

// ------------------------------------------------------------------------
class Note extends NoteRest {

    public PitchEnum pitch;
    public int octave;
    public AccidentalEnum accidental;

    public Note() {
        pitch = PitchEnum.UNDEFINED;
        octave = -1;
        accidental = AccidentalEnum.UNDEFINED;
        numerator = 0; // se nella classe NoteRest fosse stato dichiarato private, non si sarebbe potuto fare
        denominator = 1;
    }

    public Note(PitchEnum pitch, int octave, AccidentalEnum accidental, int numerator, int denominatorExp) {
        this.pitch = pitch;
        this.octave = octave;
        this.accidental = accidental;
        this.numerator = Math.abs(numerator);
        denominatorExp = Math.max(0, denominatorExp);
        denominatorExp = Math.min(denominatorExp, 8);
        this.denominator = (int) (Math.pow(2, denominatorExp));
    }

    @Override
    public String toString() {
        String stringAccidental = "";
        if (pitch == PitchEnum.UNDEFINED || octave < 0)
            return "Undefined";
        switch (accidental) {
            case SHARP:
                stringAccidental = "#";
                break;
            case FLAT:
                stringAccidental = "b";
                break;
            case DOUBLESHARP:
                stringAccidental = "x";
                break;
            case DOUBLEFLAT:
                stringAccidental = "bb";
        }
        return (pitch + stringAccidental + octave + " [" + this.getNumerator() + "/" + this.getDenominator() + "]");
    }

    @Override
    public Note clone() {
        Note n = new Note();
        n.pitch = this.pitch;
        n.octave = this.octave;
        n.accidental = this.accidental;
        n.numerator = this.numerator;
        n.denominator = this.denominator;
        return n;
    }

    public int convertToMidiPitch() {
        int result;

        result = 12 * octave;
        switch (pitch) {
            case D:
                result += 2;
                break;
            case E:
                result += 4;
                break;
            case F:
                result += 5;
                break;
            case G:
                result += 7;
                break;
            case A:
                result += 9;
                break;
            case B:
                result += 11;
                break;
            case UNDEFINED:
                return -1;
        }
        switch (accidental) {
            case SHARP:
                result += 1;
                break;
            case FLAT:
                result -= 1;
                break;
            case DOUBLESHARP:
                result += 2;
                break;
            case DOUBLEFLAT:
                result -= 2;
                break;
        }

        if (result < 0 || result > 127)
            return -1;

        return result;
    }

    public Note returnHighest(Note noteToCompare) {
        return (this.convertToMidiPitch() >= noteToCompare.convertToMidiPitch()) ? this : noteToCompare;
    }

    public void transposeGrades(int grades) {
        if (grades == 0)
            return;
        int enumIndex = pitch.ordinal();
        if (grades > 0)
            for (int i = 0; i < grades; i++) {
                enumIndex++;
                if (enumIndex > 6)
                    enumIndex = 0;
                if (enumIndex == 2)
                    octave++;
            }
        else
            for (int i = 0; i < grades; i++) {
                enumIndex--;
                if (enumIndex < 0)
                    enumIndex = 6;
                if (enumIndex == 1)
                    octave--;
            }
        pitch = PitchEnum.values()[enumIndex];
    }
}

class Melody {

    public ArrayList<NoteRest> list = new ArrayList<>(); // va istanziata!

    public int countNotes() {
        int counter = 0;
        for (NoteRest nr : list)
            if (nr instanceof Note)
                counter++;
        return counter;
    }

    public int countRests() {
        int counter = 0;
        for (NoteRest nr : list)
            if (!(nr instanceof Note))
                counter++;
        return counter;
    }

    public NoteRest returnLongest() {
        if (list.isEmpty())
            return null;

        NoteRest candidate = new NoteRest(0, 1);
        for (NoteRest nr : list)
            if (nr.getNumericDuration() > candidate.getNumericDuration())
                candidate = nr;
        return candidate;
    }

    public NoteRest returnShortest() {
        if (list.isEmpty())
            return null;

        NoteRest candidate = list.get(0);
        for (NoteRest nr : list)
            if (nr.getNumericDuration() < candidate.getNumericDuration())
                candidate = nr;
        return candidate;
    }

    public Note returnHigher() {
        if (this.countNotes() == 0)
            return null;

        Note candidate = new Note();
        for (NoteRest nr : list)
            // considero solo le note nella sequenza
            if (nr instanceof Note) {
                // se non ho inizializzato il candidato, provvedo
                if (candidate.pitch == PitchEnum.UNDEFINED)
                    candidate = ((Note) nr).clone();
                if (((Note) nr).convertToMidiPitch() > candidate.convertToMidiPitch())
                    candidate = ((Note) nr).clone();
            }
        return candidate;
    }

    public Note returnLower() {
        if (this.countNotes() == 0)
            return null;

        Note candidate = new Note();
        for (NoteRest nr : list)
            // considero solo le note nella sequenza
            if (nr instanceof Note) {
                // se non ho inizializzato il candidato, provvedo
                if (candidate.pitch == PitchEnum.UNDEFINED)
                    candidate = ((Note) nr).clone();
                if (((Note) nr).convertToMidiPitch() < candidate.convertToMidiPitch())
                    candidate = ((Note) nr).clone();
            }
        return candidate;
    }

    @Override
    public String toString() {
        String result = "";
        for (NoteRest nr : list)
            result += nr.toString() + "; ";
        return result;
    }
}

class MelodyChords {

    public ArrayList<ArrayList<Note>> list = new ArrayList<>(); // va istanziata!

    public int countTotalNotes() {
        int counter = 0;
        for (ArrayList<Note> chord : list)
            for (Note n : chord)
                counter++;
        return counter;
    }

    public NoteRest returnLongest() {
        if (list.isEmpty())
            return null;

        Note candidate = new Note(PitchEnum.C, 0, AccidentalEnum.NATURAL, 0, 1);
        for (ArrayList<Note> chord : list)
            for (Note n : chord)
                if (n.getNumericDuration() > candidate.getNumericDuration())
                    candidate = n;
        return candidate;
    }

    public NoteRest returnShortest() {
        if (list.isEmpty())
            return null;

        Note candidate = new Note(PitchEnum.C, 0, AccidentalEnum.NATURAL, Integer.MAX_VALUE, 1);
        for (ArrayList<Note> chord : list)
            for (Note n : chord)
                if (n.getNumericDuration() < candidate.getNumericDuration())
                    candidate = n;
        return candidate;
    }

    public Note returnHigher() {
        if (list.isEmpty())
            return null;

        Note candidate = new Note();
        for (ArrayList<Note> chord : list)
            for (Note n : chord) {
                // se non ho inizializzato il candidato, provvedo
                if (candidate.pitch == PitchEnum.UNDEFINED)
                    candidate = ((Note) n).clone();
                if (((Note) n).convertToMidiPitch() > candidate.convertToMidiPitch())
                    candidate = ((Note) n).clone();
            }
        return candidate;
    }

    public Note returnLower() {
        if (list.isEmpty())
            return null;

        Note candidate = new Note();
        for (ArrayList<Note> chord : list)
            for (Note n : chord) {
                // se non ho inizializzato il candidato, provvedo
                if (candidate.pitch == PitchEnum.UNDEFINED)
                    candidate = ((Note) n).clone();
                if (((Note) n).convertToMidiPitch() < candidate.convertToMidiPitch())
                    candidate = ((Note) n).clone();
            }
        return candidate;
    }
    
    public TreeMap<Integer, HashSet<ArrayList<Note>>> getChordsSet() {
        TreeMap<Integer, HashSet<ArrayList<Note>>> result = new TreeMap<>();
        for (ArrayList<Note> chord : list) {
            if (!result.containsKey(chord.size()))
                result.put(chord.size(), new HashSet<>());
            result.get(chord.size()).add(chord);
        }
        return result;
    }

    @Override
    public String toString() {
        String result = "";
        for (ArrayList<Note> chords : list) {
            for (Note n : chords)
                result += n.toString() + "; ";
            result += "| ";
        }
        return result.substring(0, result.length() - 2);
    }
}

public class TestMelodyChord {

    public static void main(String[] args) {

        MelodyChords myMelodyChord = new MelodyChords();
        Note n1 = new Note(PitchEnum.C, 4, AccidentalEnum.NATURAL, 1, 2);
        Note n2 = new Note(PitchEnum.E, 4, AccidentalEnum.NATURAL, 1, 2);
        Note n3 = new Note(PitchEnum.G, 4, AccidentalEnum.NATURAL, 1, 2); // Attenzione: nulla vieterebbe di inserire un accordo con note di durata diversa
        ArrayList<Note> chord1 = new ArrayList<>();
        chord1.add(n1);
        chord1.add(n2);
        chord1.add(n3);
        myMelodyChord.list.add(chord1);
        Note n4 = new Note(PitchEnum.G, 4, AccidentalEnum.NATURAL, 1, 3);
        Note n5 = new Note(PitchEnum.B, 4, AccidentalEnum.FLAT, 1, 3);
        ArrayList<Note> chord2 = new ArrayList<>();
        chord2.add(n4);
        chord2.add(n5);
        myMelodyChord.list.add(chord2);
        Note n6 = new Note(PitchEnum.D, 5, AccidentalEnum.NATURAL, 1, 3);
        Note n7 = new Note(PitchEnum.F, 5, AccidentalEnum.SHARP, 1, 3);
        ArrayList<Note> chord3 = new ArrayList<>();
        chord3.add(n6);
        chord3.add(n7);
        myMelodyChord.list.add(chord3);

        // Risultati
        System.out.println("Sequenza inserita: " + myMelodyChord.toString());
        System.out.println("Numero di note contenute nella melodia: " + myMelodyChord.countTotalNotes());
        System.out.println("Pitch piu' acuto: " + myMelodyChord.returnHigher().toString());
        System.out.println("Pitch piu' grave: " + myMelodyChord.returnLower().toString());
        System.out.println("Simbolo piu' lungo: " + myMelodyChord.returnLongest().toString());
        System.out.println("Simbolo piu' breve: " + myMelodyChord.returnShortest().toString());
        System.out.println("Lista di accordi ordinati per numero di note: " + myMelodyChord.getChordsSet().toString());
    }
}
